unit SampleMain;

{ Sample application to use Toolbar97 and the tb97Edit customizer dialog }

{
  Steps to using the customizer dialog for your Toolbar97 application:
   1) Include the tb97edit unit (and its form) in your app.
   2) Set up your default toolbar at design time with all possibly available buttons
      (the list of available buttons in the customizer is taken from this).
   2) In your toolbar form's Create event, call TB97InitializeToolbar(),
      passing the Toolbar component name, registry key, and help context ID.
   3) To display the dialog, simply instantiate the form, show it modally,
      then release it.
   4) In your toolbar form's Destroy event, call TB97ShutdownToolbar();
}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Menus, TB97Ctls, TB97, TB97Tlbr;

type
  TForm1 = class(TForm)
    Dock971: TDock97;
    Toolbar971: TToolbar97;
    ToolbarButton971: TToolbarButton97;
    ToolbarButton972: TToolbarButton97;
    ToolbarButton973: TToolbarButton97;
    PopupMenu1: TPopupMenu;
    Customize1: TMenuItem;
    Cancel1: TMenuItem;
    procedure Customize1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

uses tb97edit;

const
  TOOLBAR_REG_KEY = '\Software\Software with Brains, Inc.\Sample';
  TOOLBAR_HELP_ID = 0;

{$R *.DFM}

procedure TForm1.Customize1Click(Sender: TObject);
var
  ToolbarEditForm : TToolbarEditForm;
begin
  { Toolbar Edit Form is dynamically created in main app --
    Remember to remove ToolbarEditForm from
    the AutoCreate column in the project options dialog
    when dynamically creating forms.  If you want, you could
    let the application create the form at startup, but it's
    less efficient to keep it hanging around using up resources. }

  ToolbarEditForm := TToolbarEditForm.Create(Application);

  With ToolbarEditForm Do
    begin
      ShowModal;
      Release;
    end;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  { Initialize the toolbar customizer unit:
    Param #1 - The name of the TToolbar component to be customized
    Param #2 - The registry key where the toolbar configuration will be saved
               (this is always under HKEY_CURRENT_USER)
    Param #3 - The context ID for the help system page to display when the user
                presses F1 while the toolbar customizer dialog is displayed. }
  TB97InitializeToolbar(Toolbar971, TOOLBAR_REG_KEY, TOOLBAR_HELP_ID);
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  { Release all toolbar customizer unit resources -
    call when destroying your main application form. }
  TB97ShutdownToolbar;
end;

end.
