unit tb97edit;

{ *******************************************************
   Unit:     tb97Edit
   Purpose:  TToolbar97 Component Customizer Dialog

   (C)Copyright 2001 Software with Brains, Inc.
   All rights reserved.
   http://www.softwarewithbrains.com
   support@softwarewithbrains.com

   Notices:
   TToolbar97 component is (C)Copyright Jordan Russell.

   License is granted to use, modify and redistribute
   this unit in your applications as you see fit, as
   long as the copyright notices contained herein remain
   unmodified.  All ownership and copyright for this code
   shall remain with the original author,
   Software with Brains, Inc.
  *******************************************************
}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ImgList, ComCtrls, Buttons, StdCtrls, Registry, ExtCtrls,
  TB97, TB97Tlbr, TB97Ctls, TB97Tlwn;

type
  TToolbarEditForm = class(TForm)
    lstAvailable: TListView;
    btnSelect: TSpeedButton;
    btnDeselect: TSpeedButton;
    Label1: TLabel;
    Label2: TLabel;
    btnOk: TButton;
    btnCancel: TButton;
    chkShowCaptions: TCheckBox;
    lstSelected: TListView;
    imgButtons: TImageList;
    imgSeparator: TImage;
    DefaultBtn: TButton;
    procedure lstSelectedDragDrop(Sender, Source: TObject; X, Y: Integer);
    procedure lstSelectedDragOver(Sender, Source: TObject; X, Y: Integer;
      State: TDragState; var Accept: Boolean);
    procedure btnSelectClick(Sender: TObject);
    procedure lstSelectedMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure btnDeselectClick(Sender: TObject);
    procedure btnOkClick(Sender: TObject);
    procedure btnCancelClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure lstAvailableDblClick(Sender: TObject);
    procedure lstSelectedDblClick(Sender: TObject);
    procedure SetAvailableList;
    procedure SetSelectedList;
    procedure DefaultBtnClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  published
  end;

var
  ToolbarEditForm: TToolbarEditForm;
  tbToolbar : TToolbar97;
  sRegistryKey : String;
  lHelpContextID : LongInt;

procedure TB97SaveToolbarButtons(Toolbar : TToolbar97);
procedure TB97RestoreToolbarButtons(Toolbar : TToolbar97);
procedure TB97UpdateToolbarDisplay(Toolbar : TToolbar97; SelectedCommands : TStrings; ShowCaptions : Boolean);
function  TB97FindToolbarButton(Toolbar : TToolbar97; BtnCaption : String) : Integer;
procedure TB97InitializeToolbar(Toolbar : TToolbar97; RegistryKey : String; HelpContextID : LongInt);
procedure TB97ShutdownToolbar;

implementation

const
  { These may be customized to suit your application's needs
    The toolbar button widths are automatic when captions are showing,
    based on the caption widths, but that could be modified as well to
    be static. }
  SMALL_BUTTON_HEIGHT = 22;
  SMALL_BUTTON_WIDTH = 22;
  LARGE_BUTTON_HEIGHT = 44;

var
  ShowCaptions : Boolean;
  SelectedCommands : TStrings;
  DefaultCommands : TStrings;

{$R *.DFM}

procedure TToolbarEditForm.lstSelectedDragDrop(Sender, Source: TObject; X, Y: Integer);
var
  NewListItem : TListItem;
  NewPos : Integer;
begin
  If (Source is TListView) then
    With (Sender as TListView) Do
      begin
        If ((Source as TListView).Selected.Caption <> 'Separator') or (Sender <> lstAvailable) or ((Source as TListView) <> lstSelected) Then
          begin
            If (GetItemAt(X,Y) <> Nil) Then
              NewPos := GetItemAt(X,Y).Index
            Else
              NewPos := Items.Count;

            NewListItem := Items.Insert(NewPos);
            NewListItem.Caption := (Source as TListView).Selected.Caption;
            NewListItem.ImageIndex := (Source as TListView).Selected.ImageIndex;
          end;

        If (((Source as TListView).Selected.Caption <> 'Separator') or (Sender = lstAvailable)) or
           (((Source as TListView).Selected.Caption = 'Separator') and (Sender = lstSelected) and ((Source as TListView) = lstSelected)) Then
        (Source as TListView).Selected.Delete;
      end;
end;

procedure TToolbarEditForm.lstSelectedMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  (Sender as TListView).Selected := (Sender as TListView).GetItemAt(X,Y);
end;

procedure TToolbarEditForm.lstSelectedDragOver(Sender, Source: TObject; X, Y: Integer;
  State: TDragState; var Accept: Boolean);
begin
  Accept := (Source is TListView);
end;

procedure TToolbarEditForm.btnSelectClick(Sender: TObject);
begin
  If (lstAvailable.Selected <> Nil) Then
    begin
      lstSelectedDragDrop(lstSelected, lstAvailable, $FFFF, $FFFF);
    end;
end;

procedure TToolbarEditForm.btnDeselectClick(Sender: TObject);
begin
  If (lstSelected.Selected <> Nil) Then
    begin
      lstSelectedDragDrop(lstAvailable, lstSelected, $FFFF, $FFFF);
    end;
end;

procedure TToolbarEditForm.btnOkClick(Sender: TObject);
var
  I : Integer;
begin
  SelectedCommands.Clear;
  For I := 0 to lstSelected.Items.Count - 1 Do
    SelectedCommands.Add(lstSelected.Items[I].Caption);

  ShowCaptions := chkShowCaptions.Checked;

  TB97UpdateToolbarDisplay(tbToolbar, SelectedCommands, ShowCaptions);
  TB97SaveToolbarButtons(tbToolbar);
  TB97RestoreToolbarButtons(tbToolbar);

  ModalResult := mrOK;
end;

procedure TToolbarEditForm.btnCancelClick(Sender: TObject);
begin
  ModalResult := mrCancel;
end;

procedure TToolbarEditForm.DefaultBtnClick(Sender: TObject);
var
  I : Integer;
  BtnNum : Integer;
  lstItem : TListItem;
begin
  lstAvailable.Items.BeginUpdate;
  lstSelected.Items.BeginUpdate;

  lstAvailable.Items.Clear;
  lstSelected.Items.Clear;

  SetAvailableList;

  For I := 0 to DefaultCommands.Count - 1 Do
    begin
      BtnNum := TB97FindToolbarButton(tbToolbar, DefaultCommands[I]);

      If (BtnNum >= 0) Then
        begin
          lstItem := Nil;
          If (tbToolbar.Controls[BtnNum] is TToolbarButton97) Then
            lstItem := lstAvailable.FindCaption(0, (tbToolbar.Controls[BtnNum] as TToolbarButton97).Caption, False, True, False)
          Else If (tbToolbar.Controls[BtnNum] is TToolbarSep97) Then
            lstItem := lstAvailable.FindCaption(0, 'Separator', False, True, False);

          If (lstItem <> Nil) Then
            begin
              lstAvailable.Selected := lstItem;
              btnSelectClick(btnSelect);
            end;
        end;
    end;

  lstAvailable.Items.EndUpdate;
  lstSelected.Items.EndUpdate;

  chkShowCaptions.Checked := False;
end;

procedure TToolbarEditForm.SetSelectedList;
var
  lstItem : TListItem;
  I : Integer;
begin
  lstSelected.Items.BeginUpdate;

  With lstSelected Do
    begin
      Items.Clear;

      { Add the currently displayed (visible) buttons to the selected list. }
      For I := 0 to tbToolbar.ControlCount - 1 Do
        If (tbToolbar.OrderedControls[I].Visible) Then
          begin
            lstItem := Nil;
            
            If (tbToolbar.OrderedControls[I] is TToolbarButton97) Then
              lstItem := lstAvailable.FindCaption(0, (tbToolbar.OrderedControls[I] as TToolbarButton97).Caption, False, True, False)
            Else If (tbToolbar.OrderedControls[I] is TToolbarSep97) Then
              lstItem := lstAvailable.FindCaption(0, 'Separator', False, True, False);

            If (lstItem <> Nil) Then
              begin
                lstAvailable.Selected := lstItem;
                btnSelectClick(btnSelect);
              end;
          end;
    end;

  lstSelected.Items.EndUpdate;
end;

procedure TToolbarEditForm.FormCreate(Sender: TObject);
var
  Reg : TRegistry;
begin
  HelpContext := lHelpContextID;

  SelectedCommands := TStringList.Create;
  SelectedCommands.Clear;

  Reg := TRegistry.Create;
  With Reg Do
    begin
      Access := KEY_READ or KEY_WRITE;
      RootKey := HKEY_CURRENT_USER;

      If (OpenKey(sRegistryKey, False)) Then
        ShowCaptions := ReadBool('Show Captions');

      Free;
    end;

  SetAvailableList;
  SetSelectedList;

  chkShowCaptions.Checked := ShowCaptions;
end;

procedure TToolbarEditForm.FormDestroy(Sender: TObject);
begin
  If (SelectedCommands <> Nil) Then SelectedCommands.Free;
end;

procedure TToolbarEditForm.lstAvailableDblClick(Sender: TObject);
begin
  btnSelectClick(btnSelect);
end;

procedure TToolbarEditForm.lstSelectedDblClick(Sender: TObject);
begin
  btnDeselectClick(lstSelected);
end;

function TB97FindToolbarButton(Toolbar : TToolbar97; BtnCaption : String) : Integer;
var
  I : Integer;
begin
  Result := -1;

  For I := Toolbar.ControlCount - 1 DownTo 0 Do
    begin
      If ((Toolbar.Controls[I] is TToolbarSep97) and (BtnCaption = 'Separator')) Then
        begin
          Result := I;
          break;
        end
      Else If (Toolbar.Controls[I] is TToolbarButton97) Then
        begin
          If ((Toolbar.Controls[I] as TToolbarButton97).Caption = BtnCaption) Then
            begin
              Result := I;
              break;
            end;
        end;
    end;
end;

procedure TB97UpdateToolbarDisplay(Toolbar : TToolbar97; SelectedCommands : TStrings; ShowCaptions : Boolean);
var
  I : Integer;
  BtnNum : Integer;
  NewSep : TToolbarSep97;
  MinCaptionWidth : Integer;
  BtnCanvas : TCanvas;
begin
  With Toolbar Do
    begin
      BeginUpdate;

      { Remove all existing separators from the toolbar }
      For I := Toolbar.ControlCount - 1 DownTo 0 Do
        begin
          If (Toolbar.Controls[I] is TToolbarSep97) Then
            Toolbar.Controls[I].Free;
        end;

      { Start by hiding all existing buttons on the toolbar }
      For I := 0 to Toolbar.ControlCount - 1 Do
        Toolbar.Controls[I].Visible := False;

      If (ShowCaptions) Then Height := 60;

      { Find longest button caption }
      MinCaptionWidth := 46;
      BtnCanvas := (Toolbar.Controls[0] as TToolbarButton97).Canvas;
      For I := 0 To SelectedCommands.Count - 1 Do
        If (SelectedCommands[I] <> 'Separator') Then
          begin
            If (BtnCanvas.TextWidth(SelectedCommands[I]) + 4 > MinCaptionWidth) Then
              MinCaptionWidth := BtnCanvas.TextWidth(SelectedCommands[I]) + 4;
          end;

      For I := 0 To SelectedCommands.Count - 1 Do
        begin
          If (SelectedCommands[I] = 'Separator') Then
            begin
              NewSep := TToolbarSep97.Create(Toolbar);
              NewSep.Name := 'ToolbarSep97' + IntToStr(I);
              NewSep.Parent := Toolbar;
              NewSep.Width := 6;
              NewSep.Top := 0;
              NewSep.Left := 0;
              NewSep.SizeHorz := 6;
              NewSep.SizeVert := 6;
              NewSep.Blank := False;
              NewSep.Visible := True;
              OrderIndex[NewSep] := I;
            end
          Else
            begin
              BtnNum := TB97FindToolbarButton(Toolbar, SelectedCommands[I]);

              If (BtnNum >= 0) Then
                begin
                  With (Toolbar.Controls[BtnNum]) Do
                    begin
                      OrderIndex[Controls[BtnNum]] := I;
                      Visible := True;
                    end;

                  If (Toolbar.Controls[BtnNum] is TToolbarButton97) Then
                    begin
                      With (Toolbar.Controls[BtnNum] as TToolbarButton97) Do
                        begin
                          If (ShowCaptions) Then
                            begin
                              DisplayMode := dmBoth;
                              Height := LARGE_BUTTON_HEIGHT;
                              Width := MinCaptionWidth;
                            end
                          Else
                            begin
                              DisplayMode := dmGlyphOnly;
                              Height := SMALL_BUTTON_HEIGHT;
                              Width := SMALL_BUTTON_WIDTH;
                            end;
                        end;
                    end;
                end;
            end;
        end;

      EndUpdate;
    end;
end;

procedure TB97SaveToolbarButtons(Toolbar : TToolbar97);
var
  Reg : TRegistry;
  I : Integer;
  ButtonList : TStringList;
begin
  Reg := TRegistry.Create;
  With Reg Do
    begin
      Access := KEY_READ or KEY_WRITE;
      RootKey := HKEY_CURRENT_USER;

      OpenKey(sRegistryKey, True);
      WriteBool('Show Captions', ShowCaptions);

      { Delete any existing button keys first }
      OpenKey(sRegistryKey + '\Buttons', True);

      ButtonList := TStringList.Create;
      ButtonList.Clear;
      GetValueNames(ButtonList);
      For I := 0 to ButtonList.Count - 1 Do
        DeleteValue(ButtonList[I]);

      { Now add the list of currently displayed (visible) buttons only. }
      For I := 0 to Toolbar.ControlCount - 1 Do
        If (Toolbar.OrderedControls[I].Visible) Then
          begin
            If (Toolbar.OrderedControls[I] is TToolbarButton97) Then
              WriteString('Button'+Format('%.2d', [I]), (Toolbar.OrderedControls[I] as TToolbarButton97).Caption)
            Else If (Toolbar.OrderedControls[I] is TToolbarSep97) Then
              WriteString('Button'+Format('%.2d', [I]), 'Separator');
          end;

      Free;
    end;
end;

procedure TB97RestoreToolbarButtons(Toolbar : TToolbar97);
var
  Reg : TRegistry;
  I : Integer;
  ButtonList : TStringList;
  BtnCaptions : Boolean;
begin
  BtnCaptions := False;
  ButtonList := TStringList.Create;
  ButtonList.Clear;

  Reg := TRegistry.Create;
  With Reg Do
    begin
      Access := KEY_READ or KEY_WRITE;
      RootKey := HKEY_CURRENT_USER;

      If (OpenKey(sRegistryKey, False)) Then
        If (ValueExists('Show Captions')) Then
          BtnCaptions := ReadBool('Show Captions');

      If (OpenKey(sRegistryKey + '\Buttons', True)) Then
        begin
          OpenKey(sRegistryKey + '\Buttons', True);

          { Get the list of buttons to be included on the toolbar. }
          GetValueNames(ButtonList);

          { If no buttons in the registry, set up default toolbar. }
          If (ButtonList.Count = 0) Then
            begin
              ShowCaptions := False;
              TB97SaveToolbarButtons(Toolbar);
              GetValueNames(ButtonList);
            end;

          For I := 0 to ButtonList.Count - 1 Do
            begin
              { Overwrite the list item with the name of the button from the registry value. }
              ButtonList[I] := ReadString(ButtonList[I]);
            end;
        end;

      Free;
    end;

  TB97UpdateToolbarDisplay(Toolbar, ButtonList, BtnCaptions);

  If (ButtonList <> Nil) Then ButtonList.Free;
end;

procedure TToolbarEditForm.SetAvailableList;
var
  I : Integer;
  BtnItem : TListItem;
  BtnNum : Integer;
  TempImage : TBitmap;
begin
  lstAvailable.Items.BeginUpdate;

  imgButtons.Clear;
  lstAvailable.Items.Clear;

  TempImage := TBitmap.Create;
  TempImage.Width := 16;
  TempImage.Height := 16;
  TempImage.Canvas.CopyMode := cmSrcCopy;

  For I := 0 to DefaultCommands.Count - 1 Do
    begin
      BtnNum := TB97FindToolbarButton(tbToolbar, DefaultCommands[I]);

      If (tbToolbar.Controls[BtnNum] is TToolbarButton97) Then
        With (tbToolbar.Controls[BtnNum] as TToolbarButton97) Do
          begin
            { Add toolbar image to the image list. }
            TempImage.Canvas.Draw(0, 0, Glyph);
            imgButtons.AddMasked(TempImage, TempImage.TransparentColor);

            { Add item to the Available list with its image from the imagelist. }
            BtnItem := lstAvailable.Items.Add;
            BtnItem.Caption := Caption;
            BtnItem.ImageIndex := imgButtons.Count - 1;
            BtnItem.StateIndex := -1
          end;
    end;

  {Add the separator to the button list }
  TempImage.Canvas.Draw(0, 0, imgSeparator.Picture.Bitmap);
  imgButtons.AddMasked(TempImage, TempImage.TransparentColor);

  BtnItem := lstAvailable.Items.Add;
  BtnItem.Caption := 'Separator';
  BtnItem.ImageIndex := imgButtons.Count - 1;
  BtnItem.StateIndex := -1;

  lstAvailable.Items.EndUpdate;

  If (TempImage <> Nil) Then TempImage.Free;
end;

procedure TB97InitializeToolbar(Toolbar : TToolbar97; RegistryKey : String; HelpContextID : LongInt);
var
  SortList : TStringList;
  PosCaption : String;
  I : Integer;
begin
  DefaultCommands := TStringList.Create;

  tbToolbar := Toolbar;
  sRegistryKey := RegistryKey;

  SortList := TStringList.Create;
  SortList.Clear;
  SortList.Sorted := False;

  For I := 0 to tbToolbar.ControlCount - 1 Do
    begin
      If (tbToolbar.Controls[I] is TToolbarButton97) Then
        PosCaption := Format('%.4d%s', [tbToolbar.Controls[I].Left, (tbToolbar.Controls[I] as TToolbarButton97).Caption])
      Else If (tbToolbar.Controls[I] is TToolbarSep97) Then
        PosCaption := Format('%.4d%s', [tbToolbar.Controls[I].Left, 'Separator']);

      SortList.Add(PosCaption);
    end;

  SortList.Sort;

  For I := 0 to SortList.Count - 1 Do
    SortList[I] := Copy(SortList[I], 5, Length(SortList[I]));

  DefaultCommands.Clear;
  DefaultCommands.Assign(SortList);

  If (SortList <> Nil) Then SortList.Free;

  TB97RestoreToolbarButtons(tbToolbar);  
end;

procedure TB97ShutdownToolbar;
begin
  DefaultCommands.Free;
end;


end.
